#include "lpc2106_init.cpp"
#include "lpc2106_memorymap.hpp"
#include "TaltOS_base.cpp"

TaltOS tos;

TaltOS* os;

void task_1(unsigned long what);
void task_2(unsigned long what);

void timer0_irq_handler(void)
{
	tos.time->tick();
	REG8(TIMER0_IR) = 0xFF;		// Clear the interrupt flag
}

void default_irq_handler(void)
{
	REG8(UART0_THR) = '@';
}

void uart0_irq_handler(void)
{
	switch (REG8(UART0_IIR) & 0x0E) //Mask with 1110
	{
		case (0x02) :				//In case of THRE interrupt (more often interrupt)
		{
			unsigned char ch=tos.output_buffer->get_char();
			if (ch != 0)  //Do we have something to write out?
				{
					REG8(UART0_THR) = ch;
				}
			else
				{
					REG8(UART0_IER) = 0x01; //If we have nothing to write out
				}
			break;
		}
		case (0x04) :				//In case od RDA interrupt (less often interrupt)
		{
			REG8(UART0_THR) = REG8(UART0_RBR);
			tos.time->write_statistic();
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			break;
		}
	}
}

void task_1(unsigned long what)
{
	switch (what)
	{
		case 0 :
		{
			while(!tos.output_buffer->put_char('1')){}
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			break;
		}
	 	case 1 :
	 	{
			while(!tos.output_buffer->put_char('1')){}
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			while(!tos.task_control->add_task(task_2,2,5,1,1)){}
			break;
	 	}
	}

}

void task_2(unsigned long what)
{
	switch (what)
	{
		case 0 :
		{
			while(!tos.output_buffer->put_char('2')){}
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			break;
		}
		case 1 :
		{
			while(!tos.output_buffer->put_char('2')){}
			if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
			while(!tos.task_control->add_task(task_1,1,5,1,1)){}
			while(!tos.task_control->add_task(task_2,2,6,1,0)){}
			break;
	 	}
	}

}

void user_task(int i, int j)
{
	int k;
	for(k=0;k<i;k++){tos.task_control->add_task(task_1,1,5,1,1);}
	for(k=0;k<j;k++){tos.task_control->add_task(task_1,1,5,1,0);}
}


int main (void)
{
	lpc2106_startup::run();
	REG8(UART0_THR) = '*';
	lpc2106_startup::vectored_interrupts_init((unsigned long)timer0_irq_handler,
										(unsigned long) uart0_irq_handler,
										(unsigned long) default_irq_handler);
	tos.output_buffer->put_string("Start!");
	if (!(REG8(UART0_IER) & 0x02)){REG8(UART0_IER) = 0x03;}
	user_task(3,2);
	tos.start();
}
